package java.awt.image;

import java.awt.Rectangle;
import sun.awt.image.SunWritableRaster;
import java.awt.Point;

public class Raster
{
    protected SampleModel sampleModel;
    protected DataBuffer dataBuffer;
    protected int minX;
    protected int minY;
    protected int width;
    protected int height;
    protected int sampleModelTranslateX;
    protected int sampleModelTranslateY;
    protected int numBands;
    protected int numDataElements;
    protected Raster parent;
    
    private static native void initIDs();
    
    public static WritableRaster createInterleavedRaster(int n, int n2, int n3, int n4, Point point) {
        int[] array = new int[n4];
        for (int i = 0; i < n4; ++i) {
            array[i] = i;
        }
        return createInterleavedRaster(n, n2, n3, n2 * n4, n4, array, point);
    }
    
    public static WritableRaster createInterleavedRaster(int i, int n, int n2, int n3, int n4, int[] array, Point point) {
        int n5 = n3 * (n2 - 1) + n4 * n;
        DataBufferByte dataBufferByte;
        switch (i) {
            case 0: {
                dataBufferByte = new DataBufferByte(n5);
                break;
            }
            case 1: {
                dataBufferByte = new DataBufferUShort(n5);
                break;
            }
            default: {
                throw new IllegalArgumentException("Unsupported data type " + i);
            }
        }
        return createInterleavedRaster(dataBufferByte, n, n2, n3, n4, array, point);
    }
    
    public static WritableRaster createBandedRaster(int n, int n2, int n3, int i, Point point) {
        if (i < 1) {
            throw new ArrayIndexOutOfBoundsException("Number of bands (" + i + ") must be greater than 0");
        }
        int[] array = new int[i];
        int[] array2 = new int[i];
        for (int j = 0; j < i; ++j) {
            array2[array[j] = j] = 0;
        }
        return createBandedRaster(n, n2, n3, n2, array, array2, point);
    }
    
    public static WritableRaster createBandedRaster(int i, int n, int n2, int n3, int[] array, int[] array2, Point point) {
        int length = array2.length;
        if (array == null) {
            throw new ArrayIndexOutOfBoundsException("Bank indices array is null");
        }
        if (array2 == null) {
            throw new ArrayIndexOutOfBoundsException("Band offsets array is null");
        }
        int n4 = array[0];
        int n5 = array2[0];
        for (int j = 1; j < length; ++j) {
            if (array[j] > n4) {
                n4 = array[j];
            }
            if (array2[j] > n5) {
                n5 = array2[j];
            }
        }
        int n6 = n4 + 1;
        int n7 = n5 + n3 * (n2 - 1) + n;
        Object o;
        switch (i) {
            case 0: {
                o = new DataBufferByte(n7, n6);
                break;
            }
            case 1: {
                o = new DataBufferUShort(n7, n6);
                break;
            }
            case 3: {
                o = new DataBufferInt(n7, n6);
                break;
            }
            default: {
                throw new IllegalArgumentException("Unsupported data type " + i);
            }
        }
        return createBandedRaster(o, n, n2, n3, array, array2, point);
    }
    
    public static WritableRaster createPackedRaster(int i, int n, int n2, int[] array, Point point) {
        DataBufferByte dataBufferByte;
        switch (i) {
            case 0: {
                dataBufferByte = new DataBufferByte(n * n2);
                break;
            }
            case 1: {
                dataBufferByte = new DataBufferUShort(n * n2);
                break;
            }
            case 3: {
                dataBufferByte = new DataBufferInt(n * n2);
                break;
            }
            default: {
                throw new IllegalArgumentException("Unsupported data type " + i);
            }
        }
        return createPackedRaster(dataBufferByte, n, n2, n, array, point);
    }
    
    public static WritableRaster createPackedRaster(int n, int n2, int n3, int i, int n4, Point point) {
        if (i <= 0) {
            throw new IllegalArgumentException("Number of bands (" + i + ") must be greater than 0");
        }
        if (n4 <= 0) {
            throw new IllegalArgumentException("Bits per band (" + n4 + ") must be greater than 0");
        }
        if (i != 1) {
            int[] array = new int[i];
            int n6 = (1 << n4) - 1;
            int n7 = (i - 1) * n4;
            if (n7 + n4 > DataBuffer.getDataTypeSize(n)) {
                throw new IllegalArgumentException("bitsPerBand(" + n4 + ") * bands is  greater than data type size.");
            }
            switch (n) {
                case 0:
                case 1:
                case 3: {
                    for (int j = 0; j < i; ++j) {
                        array[j] = n6 << n7;
                        n7 -= n4;
                    }
                    return createPackedRaster(n, n2, n3, array, point);
                }
                default: {
                    throw new IllegalArgumentException("Unsupported data type " + n);
                }
            }
        }
        else {
            double n5 = (double)n2;
            DataBufferByte dataBufferByte;
            switch (n) {
                case 0: {
                    dataBufferByte = new DataBufferByte((int)Math.ceil(n5 / (double)(8 / n4)) * n3);
                    break;
                }
                case 1: {
                    dataBufferByte = new DataBufferUShort((int)Math.ceil(n5 / (double)(16 / n4)) * n3);
                    break;
                }
                case 3: {
                    dataBufferByte = new DataBufferInt((int)Math.ceil(n5 / (double)(32 / n4)) * n3);
                    break;
                }
                default: {
                    throw new IllegalArgumentException("Unsupported data type " + n);
                }
            }
            return createPackedRaster(dataBufferByte, n2, n3, n4, point);
        }
    }
    
    public static WritableRaster createInterleavedRaster(DataBuffer dataBuffer, int n, int n2, int n3, int n4, int[] array, Point point) {
        if (dataBuffer == null) {
            throw new NullPointerException("DataBuffer cannot be null");
        }
        if (point == null) {
            point = new Point(0, 0);
        }
        int dataType = dataBuffer.getDataType();
        PixelInterleavedSampleModel o = new PixelInterleavedSampleModel(dataType, n, n2, n4, n3, array);
        switch (dataType) {
            case 0: {
                return new SunWritableRaster(o, dataBuffer, point);
            }
            case 1: {
                return new SunWritableRaster(o, dataBuffer, point);
            }
            default: {
                throw new IllegalArgumentException("Unsupported data type " + dataType);
            }
        }
    }
    
    public static WritableRaster createBandedRaster(DataBuffer dataBuffer, int n, int n2, int n3, int[] array, int[] array2, Point origin) {
        if (dataBuffer == null) {
            throw new NullPointerException("DataBuffer cannot be null");
        }
        if (origin == null) {
            origin = new Point(0, 0);
        }
        int dataType = dataBuffer.getDataType();
        if (array2.length != array.length) {
            throw new IllegalArgumentException("bankIndices.length != bandOffsets.length");
        }
        BandedSampleModel sampleModel = new BandedSampleModel(dataType, n, n2, n3, array, array2);
        switch (dataType) {
            case 0: {
                return new SunWritableRaster(sampleModel, dataBuffer, origin);
            }
            case 1: {
                return new SunWritableRaster(sampleModel, dataBuffer, origin);
            }
            case 3: {
                return new SunWritableRaster(sampleModel, dataBuffer, origin);
            }
            default: {
                throw new IllegalArgumentException("Unsupported data type " + dataType);
            }
        }
    }
    
    public static WritableRaster createPackedRaster(DataBuffer dataBuffer, int n, int n2, int n3, int[] array, Point origin) {
        if (dataBuffer == null) {
            throw new NullPointerException("DataBuffer cannot be null");
        }
        if (origin == null) {
            origin = new Point(0, 0);
        }
        int dataType = dataBuffer.getDataType();
        SinglePixelPackedSampleModel sampleModel = new SinglePixelPackedSampleModel(dataType, n, n2, n3, array);
        switch (dataType) {
            case 0: {
                return new SunWritableRaster(sampleModel, dataBuffer, origin);
            }
            case 1: {
                return new SunWritableRaster(sampleModel, dataBuffer, origin);
            }
            case 3: {
                return new SunWritableRaster(sampleModel, dataBuffer, origin);
            }
            default: {
                throw new IllegalArgumentException("Unsupported data type " + dataType);
            }
        }
    }
    
    public static WritableRaster createPackedRaster(DataBuffer dataBuffer, int n, int n2, int n3, Point point) {
        if (dataBuffer == null) {
            throw new NullPointerException("DataBuffer cannot be null");
        }
        if (point == null) {
            point = new Point(0, 0);
        }
        int dataType = dataBuffer.getDataType();
        if (dataType != 0 && dataType != 1 && dataType != 3) {
            throw new IllegalArgumentException("Unsupported data type " + dataType);
        }
        if (dataBuffer.getNumBanks() != 1) {
            throw new RasterFormatException("DataBuffer for packed Rasters must only have 1 bank.");
        }
        MultiPixelPackedSampleModel o = new MultiPixelPackedSampleModel(dataType, n, n2, n3);
        if (dataType == 0 && (n3 == 1 || n3 == 2 || n3 == 4)) {
            return new SunWritableRaster(o, dataBuffer, point);
        }
        return new SunWritableRaster(o, dataBuffer, point);
    }
    
    public static Raster createRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        if (sampleModel == null || dataBuffer == null) {
            throw new NullPointerException("SampleModel and DataBuffer cannot be null");
        }
        if (point == null) {
            point = new Point(0, 0);
        }
        int dataType = sampleModel.getDataType();
        if ((sampleModel instanceof PixelInterleavedSampleModel)) {
            switch (dataType) {
                case 0: {
                    return new SunWritableRaster(sampleModel, dataBuffer, point);
                }
                case 1: {
                    return new SunWritableRaster(sampleModel, dataBuffer, point);
                }
                default: {
                    break;
                }
            }
        }
        else if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
            switch (dataType) {
                case 0: {
                    return new SunWritableRaster(sampleModel, dataBuffer, point);
                }
                case 1: {
                    return new SunWritableRaster(sampleModel, dataBuffer, point);
                }
                case 3: {
                    return new SunWritableRaster(sampleModel, dataBuffer, point);
                }
                default: {
                    break;
                }
            }
        }
        else if (sampleModel instanceof MultiPixelPackedSampleModel && dataType == 0 && sampleModel.getSampleSize(0) < 8) {
            return new SunWritableRaster(sampleModel, dataBuffer, point);
        }
        return new Raster(sampleModel, dataBuffer, point);
    }
    
    public static WritableRaster createWritableRaster(SampleModel sampleModel, Point point) {
        if (point == null) {
            point = new Point(0, 0);
        }
        return createWritableRaster(sampleModel, sampleModel.createDataBuffer(), point);
    }
    
    public static WritableRaster createWritableRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point origin) {
        if (sampleModel == null || dataBuffer == null) {
            throw new NullPointerException("SampleModel and DataBuffer cannot be null");
        }
        if (origin == null) {
            origin = new Point(0, 0);
        }
        int dataType = sampleModel.getDataType();
        if ((sampleModel instanceof PixelInterleavedSampleModel)) {
            switch (dataType) {
                case 0: {
                    return new SunWritableRaster(sampleModel, dataBuffer, origin);
                }
                case 1: {
                    return new SunWritableRaster(sampleModel, dataBuffer, origin);
                }
                default: {
                    break;
                }
            }
        }
        else if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
            switch (dataType) {
                case 0: {
                    return new SunWritableRaster(sampleModel, dataBuffer, origin);
                }
                case 1: {
                    return new SunWritableRaster(sampleModel, dataBuffer, origin);
                }
                case 3: {
                    return new SunWritableRaster(sampleModel, dataBuffer, origin);
                }
                default: {
                    break;
                }
            }
        }
        else if (sampleModel instanceof MultiPixelPackedSampleModel && dataType == 0 && sampleModel.getSampleSize(0) < 8) {
            return new SunWritableRaster(sampleModel, dataBuffer, origin);
        }
        return new SunWritableRaster(sampleModel, dataBuffer, origin);
    }
    
    protected Raster(SampleModel sampleModel, Point point) {
        this(sampleModel, sampleModel.createDataBuffer(), new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    protected Raster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        this(sampleModel, dataBuffer, new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    protected Raster(SampleModel sampleModel, DataBuffer dataBuffer, Rectangle rectangle, Point point, Raster parent) {
        if (sampleModel == null || dataBuffer == null || rectangle == null || point == null) {
            throw new NullPointerException("SampleModel, dataBuffer, aRegion and sampleModelTranslate cannot be null");
        }
        this.sampleModel = sampleModel;
        this.dataBuffer = dataBuffer;
        this.minX = rectangle.x;
        this.minY = rectangle.y;
        this.width = rectangle.width;
        this.height = rectangle.height;
        if (this.width <= 0 || this.height <= 0) {
            throw new RasterFormatException("negative or zero " + ((this.width <= 0) ? "width" : "height"));
        }
        if (this.minX + this.width < this.minX) {
            throw new RasterFormatException("overflow condition for X coordinates of Raster");
        }
        if (this.minY + this.height < this.minY) {
            throw new RasterFormatException("overflow condition for Y coordinates of Raster");
        }
        this.sampleModelTranslateX = point.x;
        this.sampleModelTranslateY = point.y;
        this.numBands = sampleModel.getNumBands();
        this.numDataElements = sampleModel.getNumDataElements();
        this.parent = parent;
        return;
    }
    
    public Raster getParent() {
        return this.parent;
    }
    
    public final int getSampleModelTranslateX() {
        return this.sampleModelTranslateX;
    }
    
    public final int getSampleModelTranslateY() {
        return this.sampleModelTranslateY;
    }
    
    public WritableRaster createCompatibleWritableRaster() {
        return new SunWritableRaster(this.sampleModel, new Point(0, 0));
    }
    
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        if (n <= 0 || n2 <= 0) {
            throw new RasterFormatException("negative " + ((n <= 0) ? "width" : "height"));
        }
        return new SunWritableRaster(this.sampleModel.createCompatibleSampleModel(n, n2), new Point(0, 0));
    }
    
    public WritableRaster createCompatibleWritableRaster(Rectangle rectangle) {
        if (rectangle == null) {
            throw new NullPointerException("Rect cannot be null");
        }
        return this.createCompatibleWritableRaster(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    public WritableRaster createCompatibleWritableRaster(int n, int n2, int n3, int n4) {
        return this.createCompatibleWritableRaster(n3, n4).createWritableChild(0, 0, n3, n4, n, n2, (int[])null);
    }
    
    public Raster createTranslatedChild(int n, int n2) {
        return this.createChild(this.minX, this.minY, this.width, this.height, n, n2, (int[])null);
    }
    
    public Raster createChild(int n, int n2, int width, int height, int x, int y, int[] array) {
        if (n < this.minX) {
            throw new RasterFormatException("parentX lies outside raster");
        }
        if (n2 < this.minY) {
            throw new RasterFormatException("parentY lies outside raster");
        }
        if (n + width < n || n + width > this.width + this.minX) {
            throw new RasterFormatException("(parentX + width) is outside raster");
        }
        if (n2 + height < n2 || n2 + height > this.height + this.minY) {
            throw new RasterFormatException("(parentY + height) is outside raster");
        }
        SampleModel sampleModel;
        if (array == null) {
            sampleModel = this.sampleModel;
        }
        else {
            sampleModel = this.sampleModel.createSubsetSampleModel(array);
        }
        return new Raster(sampleModel, this.getDataBuffer(), new Rectangle(x, y, width, height), new Point(this.sampleModelTranslateX + (x - n), this.sampleModelTranslateY + (y - n2)), this);
    }
    
    public Rectangle getBounds() {
        return new Rectangle(this.minX, this.minY, this.width, this.height);
    }
    
    public final int getMinX() {
        return this.minX;
    }
    
    public final int getMinY() {
        return this.minY;
    }
    
    public final int getWidth() {
        return this.width;
    }
    
    public final int getHeight() {
        return this.height;
    }
    
    public final int getNumBands() {
        return this.numBands;
    }
    
    public final int getNumDataElements() {
        return this.sampleModel.getNumDataElements();
    }
    
    public final int getTransferType() {
        return this.sampleModel.getTransferType();
    }
    
    public DataBuffer getDataBuffer() {
        return this.dataBuffer;
    }
    
    public SampleModel getSampleModel() {
        return this.sampleModel;
    }
    
    public Object getDataElements(int n, int n2, Object o) {
        return this.sampleModel.getDataElements(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, o, this.dataBuffer);
    }
    
    public Object getDataElements(int n, int n2, int n3, int n4, Object o) {
        return this.sampleModel.getDataElements(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, o, this.dataBuffer);
    }
    
    public int[] getPixel(int n, int n2, int[] array) {
        return this.sampleModel.getPixel(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, array, this.dataBuffer);
    }
    
    public float[] getPixel(int n, int n2, float[] array) {
        return this.sampleModel.getPixel(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, array, this.dataBuffer);
    }
    
    public double[] getPixel(int n, int n2, double[] array) {
        return this.sampleModel.getPixel(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, array, this.dataBuffer);
    }
    
    public int[] getPixels(int n, int n2, int n3, int n4, int[] array) {
        return this.sampleModel.getPixels(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, array, this.dataBuffer);
    }
    
    public float[] getPixels(int n, int n2, int n3, int n4, float[] array) {
        return this.sampleModel.getPixels(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, array, this.dataBuffer);
    }
    
    public double[] getPixels(int n, int n2, int n3, int n4, double[] array) {
        return this.sampleModel.getPixels(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, array, this.dataBuffer);
    }
    
    public int getSample(int n, int n2, int n3) {
        return this.sampleModel.getSample(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, this.dataBuffer);
    }
    
    public float getSampleFloat(int n, int n2, int n3) {
        return this.sampleModel.getSampleFloat(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, this.dataBuffer);
    }
    
    public double getSampleDouble(int n, int n2, int n3) {
        return this.sampleModel.getSampleDouble(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, this.dataBuffer);
    }
    
    public int[] getSamples(int n, int n2, int n3, int n4, int n5, int[] array) {
        return this.sampleModel.getSamples(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, n5, array, this.dataBuffer);
    }
    
    public float[] getSamples(int n, int n2, int n3, int n4, int n5, float[] array) {
        return this.sampleModel.getSamples(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, n5, array, this.dataBuffer);
    }
    
    public double[] getSamples(int n, int n2, int n3, int n4, int n5, double[] array) {
        return this.sampleModel.getSamples(n - this.sampleModelTranslateX, n2 - this.sampleModelTranslateY, n3, n4, n5, array, this.dataBuffer);
    }
    
    static {
        ColorModel.loadLibraries();
        initIDs();
    }
}
